<?php

class Easy_weather_widget extends WP_Widget {

  private $refresh_rate = 300;
  private $transient_name;
  private $args;
  private $instance;
  private $text_domain = 'easy_weather_widget'; 

	public function __construct(){
    $this->text_domain = 'easy_weather_widget';
		$widget_id = Eww_helper::$widget_id;
    parent::__construct( $widget_id, 'Easy Weather', array( 'description' => __( 'Display weather information', $this->text_domain ) ) );
    

  }

	public function widget( $args, $instance ){
		if( empty( $instance['zip'] ) ) {
			return;
		}
    $this->args = $args;
    $this->instance = $instance;
    echo $this->generate_output();
  }

	public function update( $new_instance, $old_instance ){
		$instance = $old_instance;
    $instance['zip'] = $new_instance['zip'];
    $instance['temperature_scale'] = $new_instance['temperature_scale'];
    $instance['title'] = $new_instance['title'];
    if ( ! Eww_helper::is_zip( $new_instance['zip'] ) ){
      $instance['zip'] = '';
      $instance['error'] = __( 'Invalid zip code (should be 5 digits)', $this->text_domain );
    } elseif ( ! Weather_information::exists( $new_instance['zip'] ) ) {
      $instance['zip'] = '';
      $instance['error'] = __( 'Zip code does not exist', $this->text_domain );
    } else {
      $instance['error'] = '';
    }
		return $instance;
	}

	public function form( $instance ){
    $zip = $error = $zip = $error = $temperature_scale = $title = '';
    $defaults = array(
                      'zip' => '',
                      'error'=> '',
                      'temperature_scale' => 'F',
                      'title'=>'Weather'
                );
    $instance 	= wp_parse_args( (array) $instance, $defaults );
    extract( $instance );
    ?><div><?php
    if ( $error ) :
      ?><p class="eww-error"><?php echo $error; ?></p><?php
    endif;
    ?></div><?php
    ?>
    <p>
      <label for="<?php echo $this->get_field_id( 'title' ); ?>"><?php _e( "Title", $this->text_domain ); ?></label>
      <input type="text" class="widefat" id="<?php echo $this->get_field_id( 'title' )?>" name="<?php echo $this->get_field_name( 'title' ); ?>" value="<?php echo $title;?>">
    </p>
    <p>
      <label for="<?php echo $this->get_field_id( 'zip' ); ?>"><?php _e( "Zip Code", $this->text_domain ); ?></label>
      <input type="text" class="widefat" id="<?php echo $this->get_field_id( 'zip' )?>" name="<?php echo $this->get_field_name( 'zip' ); ?>" value="<?php echo $zip;?>">
    </p>
    <p>
      <label for="<?php echo $this->get_field_id( 'temperature_scale' ); ?>"><?php _e( "Temperature Scale", $this->text_domain ); ?></label>
      <select id="<?php echo $this->get_field_id( 'temperature_scale' ); ?>" name="<?php echo $this->get_field_name( 'temperature_scale' ); ?>" class="widefat" style="width:100%;">
        <option value="F" <?php if ( 'F' == $instance['temperature_scale'] ) echo 'selected="selected"'; ?>><?php _e( "Fahrenheit", $this->text_domain ); ?></option>
        <option value="C" <?php if ( 'C' == $instance['temperature_scale'] ) echo 'selected="selected"'; ?>><?php _e( "Celsius", $this->text_domain ); ?></option>
        <option value="K" <?php if ( 'K' == $instance['temperature_scale'] ) echo 'selected="selected"'; ?>><?php _e( "Kelvin", $this->text_domain ); ?></option>
      </select>
    </p>

  <?php
	}

  private function generate_output( $override = false ){
    $transient_key = $this->id;
    $weather_output = get_transient($transient_key );
    if ( false === $weather_output || true == $override ) {
      $title = apply_filters( 'widget_title', $this->instance['title'] );
      $zip = $this->instance['zip'];
      $temperature_scale = $this->instance['temperature_scale'];
      $weather_info = Weather_information::get( $zip );
      $degree_marker = apply_filters( 'eww_degree_symbol', '&#176;' );
      $city = $weather_info->city;
      $state = $weather_info->state;
      $image = $weather_info->image;
      $description = $weather_info->weather_description;
      $temperature = $weather_info->temperature;
      if ( $state ) :
        $state = ', ' . $state;
      else:
        $state = '';
      endif;
      if ( $temperature_scale != 'F' ) {
        $temperature = Eww_helper::convert_temperature( $temperature, $temperature_scale );
      }
      $temperature = $temperature . $degree_marker;
      ob_start();
        echo $this->args['before_widget'];
        if ( ! empty( $title ) ) {
          echo $this->args['before_title'] . $title . $this->args['after_title'];
        }
      ?><div class="clearfix outer-wrap"><?php
        ?><ul><li><?php
        ?><p class="bold title"><?php echo $city . $state; ?></p><?php
        ?><div class="image"><?php
          if ( $image ):
            ?><p><img src="<?php echo $image; ?>" alt="<?php echo $description; ?>"></p><?php
          endif;
        ?></div><?php
        ?><div class="data"><?php
          ?><p class="bold"><?php _e( "Currently", $this->text_domain ); ?></p><?php
          ?><p><?php _e( $description, $this->text_domain ); ?></p><?php
          ?><p><?php echo $temperature; ?></p><?php
        ?></div><?php
        ?></li></ul><?php
      ?></div><?php
        echo $this->args['after_widget'];
      $weather_output = ob_get_clean();
      set_transient( $transient_key, $weather_output, $this->refresh_rate);
    }
    return  $weather_output;
  }
}